@@ -32,21 +32,42 @@ class ActionBase(object):
     def render(self, serializer: Serializer, **kwds):
         raise NotImplementedError()
 
+    @staticmethod
+    def prepare_string(string, encode=True):
+        """
+        Replaces curly brackets with some special string so there is no problem when string.format() is called
+
+        :param string: String in which program searches for curly brackets
+        :param encode: True: replaces brackets with special string, False: the other way around
+        :return:
+        """
+        replaces = {'{': '´|curl_brack_start|`', '}': '´|curl_brack_end|`'}
+        if not encode:
+            replaces = dict([(value, key) for key, value in replaces.items()])
+
+        for key, val in replaces.items():
+            string = string.replace(key, val)
+        return string
+
 
 class RenderableActionMixin(object):
     """
     Action that is rendered on screen
     """
 
-    def __init__(self, label: str, title: str, icon: str = None):
+    def __init__(self, label: str, title: str, icon: str = None, btn_classes: Union[str, dict, None] = None):
         """
         :param label: Label for rendering to on screen control
         :param title: Hint text for on-screen control
         :param icon: optional icon to add to render
+        :param btn_classes: optional class(es) of button. if variable is dict and key 'replace' have value True,
+         then default class will be replaced with class(es) that are under key 'classes'. In other case class(es) will
+         be just added to default class
         """
         self.label = label
         self.title = title
         self.icon = icon
+        self.btn_classes = btn_classes
 
 
 class TablePosition(IntEnum):
@@ -65,21 +86,22 @@ class TableAction(ActionBase, RenderableActionMixin):
 
     def __init__(self, position: TablePosition, label: str, action_js: str,
                  title: Union[str, None] = None, icon: Union[str, None] = None, field_name: Union[str, None] = None,
-                 name: Union[str, None] = None, serializer: Serializer = None):
+                 name: Union[str, None] = None, serializer: Serializer = None,
+                 btn_classes: Union[str, dict, None] = None):
         ActionBase.__init__(self, action_js, name, serializer)
-        RenderableActionMixin.__init__(self, label, title, icon)
+        RenderableActionMixin.__init__(self, label, title, icon, btn_classes)
         self.position = position
         self.field_name = field_name
 
     def copy_and_resolve_reference(self, serializer: Serializer):
         return TableAction(self.position, self.label, self.action_js, self.title, self.icon, self.field_name, self.name,
-                           serializer)
+                           serializer, self.btn_classes)
 
     def render(self, serializer: Serializer, **kwds):
         ret = rowclick = rowrclick = ''
         stop_propagation = 'if(event.stopPropagation){event.stopPropagation();}event.cancelBubble=true;'
 
-        action_action = self.action_js
+        action_action = self.prepare_string(self.action_js)
         if self.position != TablePosition.HEADER:
             # We need to do this differently because of dynamic page loading for tables: each time the serializer
             # has a different UUID
@@ -95,12 +117,29 @@ class TableAction(ActionBase, RenderableActionMixin):
         else:
             from uuid import uuid1
 
+            def get_btn_class(default_class, additional_class):
+                classes = default_class.split(' ')
+                if additional_class:
+                    if isinstance(additional_class, dict):
+                        if additional_class.get('replace', False):
+                            classes = []
+                        additional_class = additional_class.get('classes', '').split(' ')
+                    else:
+                        additional_class = additional_class.split(' ')
+                    classes.extend(additional_class)
+                return ' '.join(classes)
+
+            button_name = (' name="btn-%s" ' % self.name) if self.name else ''
+            btn_class = get_btn_class("btn btn-info", self.btn_classes)
+
             btnid = uuid1()
-            ret += '<button id="df-action-btn-{btnid}" type="button" class="btn btn-info" ' \
+            ret += '<button id="df-action-btn-{btnid}" type="button" class="{btn_class}" title="{title}"{button_name}' \
                    'onClick="{stop_propagation} {action}">{icon_def}{label}</button>'. \
                 format(btnid=btnid, stop_propagation=stop_propagation, action=action_action,
-                       label=self.label,
-                       icon_def='<img src="{icon}"/>'.format(icon=self.icon) if self.icon else '')
+                       btn_class=btn_class, label=self.prepare_string(self.label),
+                       title=self.prepare_string(self.title),
+                       icon_def='<img src="{icon}"/>'.format(icon=self.icon) if self.icon else '',
+                       button_name=button_name)
             if DYNAMICFORMS.jquery_ui:
                 ret += '<script type="application/javascript">$("#df-action-btn-{btnid}").button();</script>' \
                     .format(btnid=btnid)
@@ -122,7 +161,7 @@ class TableAction(ActionBase, RenderableActionMixin):
                 ret=ret, direction='left' if self.position == TablePosition.FIELD_START else 'right'
             )
 
-        return ret
+        return self.prepare_string(ret, False)
 
 
 class FieldChangeAction(ActionBase):
@@ -215,10 +254,10 @@ class FormButtonAction(ActionBase):
         self.button_is_primary = button_is_primary
 
         self.btn_classes = btn_classes or (
-            DYNAMICFORMS.form_button_classes + ' '
-            + (DYNAMICFORMS.form_button_classes_primary if button_is_primary
-               else DYNAMICFORMS.form_button_classes_secondary) + ' '
-            + (DYNAMICFORMS.form_button_classes_cancel if btn_type == FormButtonTypes.CANCEL else '')
+                DYNAMICFORMS.form_button_classes + ' '
+                + (DYNAMICFORMS.form_button_classes_primary if button_is_primary
+                   else DYNAMICFORMS.form_button_classes_secondary) + ' '
+                + (DYNAMICFORMS.form_button_classes_cancel if btn_type == FormButtonTypes.CANCEL else '')
         )
 
     def copy_and_resolve_reference(self, serializer):
@@ -229,7 +268,9 @@ class FormButtonAction(ActionBase):
         if self.btn_type == FormButtonTypes.CANCEL and position == 'form':
             return ''
         action_js = self.action_js
+        button_name = ('name="btn-%s"' % self.name) if self.name else ''
         if isinstance(action_js, str):
+            action_js = self.prepare_string(action_js)
             action_js = action_js.format(**locals())
         button_type = 'button' if self.btn_type != FormButtonTypes.SUBMIT or position == 'dialog' else 'submit'
         data_dismiss = 'data-dismiss="modal"' if self.btn_type == FormButtonTypes.CANCEL else ''
@@ -246,8 +287,9 @@ class FormButtonAction(ActionBase):
         else:
             button_js = ''
 
-        return '<button type="{button_type}" class="{self.btn_classes}" {data_dismiss} id="{button_id}">' \
-               '{self.label}</button>{button_js}'.format(**locals())
+        return self.prepare_string(
+            '<button type="{button_type}" class="{self.btn_classes}" {data_dismiss} {button_name} id="{button_id}">'
+            '{self.label}</button>{button_js}'.format(**locals()), False)
 
 
 class Actions(object):
