@@ -75,16 +75,34 @@ dynamicforms = {
       $("#df-progress-bar-determinate").progressbar(progressSettings);
 
       if (dynamicforms.shouldShowProgressDlg) {
+        dynamicforms.shouldCloseProgressDlg = false;
         dynamicforms.progressDlgShown = true;
-        progressDlg.dialog({dialogClass: 'progress-bar', closeOnEscape: false, resizable: false});
+        dynamicforms.progressDlgStartShowing = true;
+        progressDlg.dialog({dialogClass: 'progress-bar', closeOnEscape: false, resizable: false,
+                            open: function() {
+                              dynamicforms.progressDlgShown = true;
+                              if (dynamicforms.shouldCloseProgressDlg) {
+                                dynamicforms.closeProgressDlg(progressDlgID);
+                              }
+                            }
+                           });
       }
       zIndexElement = progressDlg.parents().first();
     } else {
       if (progressSettings === undefined)
         progressSettings = {keyboard: false, backdrop: 'static'};
       if (dynamicforms.shouldShowProgressDlg) {
-        dynamicforms.progressDlgShown = true;
+        dynamicforms.shouldCloseProgressDlg = false;
+        dynamicforms.progressDlgStartShowing = true;
         progressDlg.modal(progressSettings);
+        progressDlg.on('shown.bs.modal', function () {
+          progressDlg.off('shown.bs.modal');
+          dynamicforms.progressDlgShown = true;
+          if(dynamicforms.shouldCloseProgressDlg){
+            dynamicforms.closeProgressDlg(progressDlgID);
+          }
+        })
+
       }
       zIndexElement = progressDlg;
     }
@@ -110,7 +128,7 @@ dynamicforms = {
    * @param progressSettings: Progress dialog settings - for custom progress dialogs.
    */
   progressCheck:         function progressCheck(progressDlgID, timestamp, progressSettings) {
-    if (!dynamicforms.progressDlgShown && dynamicforms.progressCheckInterval != null) {
+    if (!dynamicforms.progressDlgStartShowing && dynamicforms.progressCheckInterval != null) {
       clearInterval(dynamicforms.progressCheckInterval);
       dynamicforms.progressCheckInterval = null;
     } else {
@@ -118,8 +136,17 @@ dynamicforms = {
         .done(function (data, textStatus, jqXHR) {
           var pb_indet   = $('#df-progress-bar-indeterminate');
           var pb_det     = $('#df-progress-bar-determinate');
-          var percent    = data['value'];
+          var percent    = data.value;
           var show_indet = percent == null;
+          pb_indet.toggle(show_indet);
+          pb_det.toggle(!show_indet);
+
+          if (data.comment != null)
+            $('#df-progress-comment').html(data.comment);
+
+          if (dynamicforms.shouldShowProgressDlg && !dynamicforms.progressDlgStartShowing) {
+            dynamicforms.showProgressDlg(progressDlgID, progressSettings);
+          }
           if (!show_indet) {
             if (dynamicforms.DYNAMICFORMS.jquery_ui) {
               pb_det.progressbar("value", parseInt(percent));
@@ -127,15 +154,9 @@ dynamicforms = {
               pb_det.css('width', percent + '%').attr('aria-valuenow', percent)
             }
           }
-          pb_indet.toggle(show_indet);
-          pb_det.toggle(!show_indet);
-
-          if (dynamicforms.shouldShowProgressDlg && !dynamicforms.progressDlgShown) {
-            dynamicforms.showProgressDlg(progressDlgID, progressSettings);
-          }
         })
         .fail(function (jqXHR, textStatus, errorThrown) {
-          if (dynamicforms.shouldShowProgressDlg && !dynamicforms.progressDlgShown) {
+          if (dynamicforms.shouldShowProgressDlg && !dynamicforms.progressDlgStartShowing) {
             dynamicforms.showProgressDlg(progressDlgID, progressSettings);
           }
         });
@@ -152,7 +173,9 @@ dynamicforms = {
     dynamicforms.progressCheckInterval = setInterval(dynamicforms.progressCheck, 500, progressDlgID, timestamp, progressSettings);
   },
   shouldShowProgressDlg: false,
+  shouldCloseProgressDlg: true,
   progressDlgShown:      false,
+  progressDlgOverlay:    false,
   /**
    * Sets overlay so nothing can be clicked while operation last. If it last for more than 0.5 second it starts progress checker.
    * @param progressDlgID: ID of progress dialog element - for custom progress dialogs.
@@ -160,10 +183,11 @@ dynamicforms = {
    * @param progressSettings: Progress dialog settings - for custom progress dialogs.
    */
   setProgressDlg:        function setProgressDlg(progressDlgID, timestamp, progressSettings) {
+    dynamicforms.progressDlgOverlay = true;
     $('#df-overlay').css('z-index', 10000).show();
     dynamicforms.shouldShowProgressDlg = true;
     window.setTimeout(function () {
-      if (dynamicforms.shouldShowProgressDlg) {
+      if (dynamicforms.shouldShowProgressDlg && !dynamicforms.progressDlgShown) {
         dynamicforms.startProgressChecker(progressDlgID, timestamp, progressSettings);
       }
     }, 500);
@@ -173,21 +197,25 @@ dynamicforms = {
    * @param progressDlgID: ID of progress dialog element - for custom progress dialogs.
    */
   closeProgressDlg:      function closeProgressDlg(progressDlgID) {
-    dynamicforms.shouldShowProgressDlg = false
+    dynamicforms.shouldShowProgressDlg = false;
+    dynamicforms.shouldCloseProgressDlg = true;
     if (dynamicforms.progressCheckInterval != null) {
       clearInterval(dynamicforms.progressCheckInterval);
       dynamicforms.progressCheckInterval = null;
     }
-
     $('#df-overlay').hide();
     if (dynamicforms.progressDlgShown) {
       dynamicforms.progressDlgShown = false;
+      dynamicforms.progressDlgStartShowing = false;
+
+      var $dlg = $('#' + progressDlgID);
       if (dynamicforms.DYNAMICFORMS.jquery_ui) {
-        $('#' + progressDlgID).dialog('close');
+        $dlg.dialog('close');
       } else {
-        $('#' + progressDlgID).modal('hide');
+        $dlg.modal('hide');
       }
     }
+    dynamicforms.progressDlgOverlay = false;
   },
   /**
    * Calls standard jQuery.ajax. Additionally it sets overlay that prevents clicks on other elements until operation completes
@@ -200,15 +228,27 @@ dynamicforms = {
     var progressDlgID = options['progress_id'] !== undefined ? options['progress_id'] : 'df-progress-bar-container';
     var timestamp     = $.now()
 
-    dynamicforms.setProgressDlg(progressDlgID, timestamp, options['progress_setts']);
+    var showProgress  = undefined;
+    var progressSetts = options.progress_setts;
+
+    if (progressSetts != undefined)
+      showProgress = progressSetts.show;
+    if (showProgress == undefined)
+      showProgress = true;
+
+    if (showProgress && !dynamicforms.progressDlgOverlay) {
+      dynamicforms.setProgressDlg(progressDlgID, timestamp, options['progress_setts']);
+    }
+
     var closeProgressDialogFunc = function () {
-      dynamicforms.closeProgressDlg(progressDlgID);
+      if (showProgress)
+        dynamicforms.closeProgressDlg(progressDlgID);
     };
 
     var ajaxSettings          = options['ajax_setts'] !== undefined ? options['ajax_setts'] : {};
     var headers               = ajaxSettings['headers'] !== undefined ? ajaxSettings['headers'] : {};
     headers['X-DF-TIMESTAMP'] = timestamp;
-    ajaxSettings['headers'] = headers;
+    ajaxSettings['headers']   = headers;
 
     return $.ajax(ajaxSettings).done(closeProgressDialogFunc).fail(closeProgressDialogFunc);
   },
@@ -220,10 +260,11 @@ dynamicforms = {
    * @param $form: the edited form containing the data
    * @param refreshType: how to refresh the table
    * @param doneFunc: if specified, this function will be called on successful data send
+   * @param dType: Custom dataType
    */
-  submitForm: function submitForm($dlg, $form, refreshType, doneFunc) {
+  submitForm: function submitForm($dlg, $form, refreshType, doneFunc, dType) {
     var data    = dynamicforms.getSerializedForm($form, 'final');
-    var method  = data['data-dynamicforms-method'] || 'POST';
+    var method  = data['data-dynamicforms-method'] || ([0, "", "0", undefined, null].indexOf(data.id) > -1 ? 'POST' : 'PUT');
     var headers = {'X-DF-RENDER-TYPE': 'dialog'};
 
     headers['X-CSRFToken'] = dynamicforms.csrf_token;
@@ -234,22 +275,35 @@ dynamicforms = {
     var listId = dynamicforms.form_helpers.get($form.attr('id'), 'listID');
 
     var dataType = 'html';
-    if (doneFunc == undefined) {
-      doneFunc = function (data) {
-        //var formContent = $dlg.find("form").html();
-        dynamicforms.closeDialog($dlg);
-        if (!recordID) {
-          try {
-            recordID = $(data).find('form.dynamicforms-form').find('input[name=\'id\']').val().trim();
-            if (recordID == '')
-              recordID = false;
-          } catch (e) {}
-        }
-        dynamicforms.refreshList(recordURL, recordID, refreshType, listId);
+    function performRefresh(data, params){
+      if (!recordID) {
+        try {
+          recordID = $(data).find('form.dynamicforms-form').find('input[name=\'id\']').val().trim();
+          if (recordID == '')
+            recordID = false;
+        } catch (e) {}
       }
-    } else
+      dynamicforms.refreshList(recordURL, recordID, refreshType, listId, false, params);
+    }
+    var doneFuncExec;
+    if (doneFunc == undefined) {
+      doneFuncExec = function (data) {
+        dynamicforms.closeDialog($dlg);
+        performRefresh(data);
+      }
+    } else{
+      doneFuncExec = doneFunc;
       dataType = 'json';  // This is a brazen assumption that custom done functions will only ever work with JSON
+    }
+    if (dType != undefined)
+      dataType = dType;
 
+    function getDlg(){
+      return $dlg;
+    }
+    // We need this so ve can call refresh from custom function (arguments.callee.performRefresh())
+    doneFuncExec.performRefresh = performRefresh;
+    doneFuncExec.getDlg = getDlg;
     dynamicforms.ajaxWithProgress({
                                     ajax_setts: {
                                       type:        method,
@@ -260,13 +314,14 @@ dynamicforms = {
                                       traditional: true,
                                     }
                                   })
-      .done(doneFunc)
+      .done(doneFuncExec)
       .fail(function (xhr, status, error) {
         // TODO: this doesn't handle errors correctly: if return status is 400 something, it *might* be OK
         //  but if it's 500 something, dialog will be replaced by non-dialog code and displaying it will fail
         //  also for any authorization errors, CSRF, etc, it will again fail
         //  Try finding a <div class="dynamicforms-dialog"/> in there to see if you actually got a dialog
-        dynamicforms.updateDialog($dlg, $(xhr.responseText), refreshType, listId);
+        if ($dlg != null)
+          dynamicforms.updateDialog($dlg, $(xhr.responseText), refreshType, listId, doneFunc, dType);
       });
   },
 
@@ -298,28 +353,34 @@ dynamicforms = {
    * @param refreshType: how to refresh the table
    * @param formID: id of form
    * @param deletion: if present delete action happened
+   * @param params: dict variable for additional parameters
    */
-  refreshList: function refreshList(url, recordID, refreshType, formID, deletion) {
+  refreshList: function refreshList(url, recordID, refreshType, formID, deletion, params) {
     if (refreshType == undefined || refreshType == 'record') {
-      if (recordID) {
-        var trSelector = "tr[data-id='" + recordID + "']";
-        $rowToRefresh  = $(trSelector); // Row to refresh
+
+      var data = dynamicforms.filterData(formID, true);
+      if (recordID)
+        data.id = recordID
+      if (params != undefined && params.additionalData != undefined) {
+        for (var key in params.additionalData) {
+          data[key] = params.additionalData[key];
+        }
       }
 
-      // If table will not change we wont even fetch new record
-      if (!dynamicforms.isLinkNext(dynamicforms.df_tbl_pagination.get(formID, undefined).link_next) ||
-        ($rowToRefresh != null && $rowToRefresh.length)) {
-        var data = dynamicforms.filterData(formID, true);
-        if (recordID)
-          data.id = recordID
-        dynamicforms.ajaxWithProgress({ajax_setts: {type: 'GET', url: url, data: data, dataType: 'html'}})
-          .done(function (data) {
-            dynamicforms.refreshRow(data, formID, recordID);
-          })
-          .fail(function (xhr, status, error) {
-            // TODO: this doesn't handle errors correctly
-          });
-      }
+      dynamicforms.ajaxWithProgress({
+                                      ajax_setts:     {type: 'GET', url: url, data: data, dataType: 'html',
+                                        headers: {'X-DF-CALLTYPE': 'refresh_record'}},
+                                      progress_setts: params != undefined ? params.progressSetts : undefined
+                                    })
+        .done(function (data) {
+          dynamicforms.refreshRow(data, formID, recordID);
+          // noinspection JSUnresolvedVariable
+          if(params && params.afterRefreshFunc)
+            params.afterRefreshFunc(data, formID, recordID);
+        })
+        .fail(function (xhr, status, error) {
+          // TODO: this doesn't handle errors correctly
+        });
     } else if (refreshType == 'table') {
       dynamicforms.refreshTable(formID, recordID);
       if (deletion == true) {
@@ -344,14 +405,30 @@ dynamicforms = {
    * Insert new row after the last row or insert first row
    * @param $newRow
    */
-  insertRow:   function insertRow($newRow) {
+  insertRow:   function insertRow($newRow, formID, prevId) {
     // Insert new row after the last row or insert first row
     if ($newRow.length) {
-      var $lastRow = $("tr[data-id]").last(); // Last row before adding new record
-      if ($lastRow.length) {
-        $newRow.insertAfter($lastRow);
+      var tbl_pagination = dynamicforms.df_tbl_pagination.get(formID, undefined);
+
+      var $lastRow;
+      var hasPrevId = (typeof prevId !== typeof undefined && prevId !== false && prevId != '');
+
+      if(hasPrevId){
+        if(prevId == 'None')
+          $lastRow = $('#list-' + formID + ' > tbody > tr[data-id]').first();
+        else{
+          $lastRow = $('#list-' + formID + ' > tbody > tr[data-id="' + prevId + '"]').first(); // First row of table
+        }
+      }
+      else if(!dynamicforms.isLinkNext(tbl_pagination.link_next))
+        $lastRow = $('#list-' + formID + ' > tbody > tr[data-id]').last(); // Last row before adding new record
+      if ($lastRow != undefined && $lastRow.length) {
+        if(prevId == 'None')
+          $newRow.insertBefore($lastRow);
+        else
+          $newRow.insertAfter($lastRow);
       } else {
-        $("table").find("tr[data-title]").replaceWith($newRow);
+        $('#list-' + formID + ' > tbody > tr[data-title]').replaceWith($newRow);
       }
     }
   },
@@ -362,33 +439,35 @@ dynamicforms = {
    * @param formID: id of form
    */
   refreshRow:  function refreshRow(data, formID, recordID) {
-    var tbl_pagination = dynamicforms.df_tbl_pagination.get(formID, undefined);
-
     var $rowToRefresh = null; // Row to refresh
+    var $htmlObject = (data instanceof jQuery) ? data : $(data);
 
     if (recordID) {
       var trSelector = "tr[data-id='" + recordID + "']";
-      $rowToRefresh  = $(trSelector); // Row to refresh
+      $rowToRefresh  = $('#list-' + formID + ' > tbody > ' + trSelector); // Row to refresh
+      var $editedRow = $htmlObject.find("table[id^='list-'] > tbody > " + trSelector); // Edited record from ajax returned html
     }
 
-    // Case when all records are loaded or edited record is loaded
-    if (!dynamicforms.isLinkNext(tbl_pagination.link_next) || ($rowToRefresh != null && $rowToRefresh.length)) {
-      var $htmlObject = $(data);
+    if (recordID) {
+      if ($editedRow.length) {
+        var prevId = $editedRow.attr('data-df_prev_id');
+        var hasPrevId = (typeof prevId !== typeof undefined && prevId !== false && prevId != '');
 
-      if (recordID) {
-        var $editedRow = $htmlObject.find(trSelector); // Edited record from ajax returned html
-        if ($editedRow.length) {
-          if ($rowToRefresh.length)
-            $rowToRefresh.replaceWith($editedRow);
+        if ($rowToRefresh != null && $rowToRefresh.length){
+          if(hasPrevId){
+            $rowToRefresh.remove();
+            dynamicforms.insertRow($editedRow, formID, prevId);
+          }
           else
-            dynamicforms.insertRow($editedRow);
+            $rowToRefresh.replaceWith($editedRow);
         }
-      } else {
-        var $newRow = $htmlObject.find("table").find("tr[data-id]").last(); // Added record from ajax returned html
-        dynamicforms.insertRow($newRow);
+        else
+          dynamicforms.insertRow($editedRow, formID, prevId);
       }
+    } else {
+      var $newRow = $htmlObject.find("table").find("tr[data-id]").last(); // Added record from ajax returned html
+      dynamicforms.insertRow($newRow, formID, $newRow.attr('data-df_prev_id'));
     }
-    // else do nothing. Paginator will take care of new record when necessary
   },
 
   /**
@@ -408,8 +487,10 @@ dynamicforms = {
    * @param $dlg: dialog (parsed) html to show
    * @param refreshType: how to refresh the table after the dialog is finished with editing
    * @param listId: id of table element with records
+   * @param doneFunc: if specified, this function will be called on successful data send
+   * @param dataType: Custom dataType
    */
-  showDialog: function showDialog($dlg, refreshType, listId) {
+  showDialog: function showDialog($dlg, refreshType, listId, doneFunc, dataType) {
     //TODO: adjust hashURL
     $(document.body).append($dlg);
     var $form = $dlg.find('.dynamicforms-form');
@@ -427,17 +508,12 @@ dynamicforms = {
 
     var saveId = '#save-' + $form.attr('id');
     $(saveId).on('click', function () {
-      dynamicforms.submitForm($dlg, $form, refreshType);
+      dynamicforms.submitForm($dlg, $form, refreshType, doneFunc, dataType);
     });
     // And show the dialog
     if (dynamicforms.DYNAMICFORMS.jquery_ui)
       $dlg.dialog('open')
     else {
-      $dlg.on('hidden.bs.modal', function () {
-        $dlg.remove();
-        if ($dlg.showNewAfterHide)
-          dynamicforms.showDialog($dlg.showNewAfterHide, refreshType, listId);
-      });
       $dlg.modal();
     }
   },
@@ -448,10 +524,12 @@ dynamicforms = {
    * @param $newDlg: new dialog with errors in form
    * @param refreshType: how to refresh the table after the dialog is finished with editing
    * @param listId: id of table element with records
+   * @param doneFunc: if specified, this function will be called on successful data send
+   * @param dType: Custom dataType
    */
-  updateDialog: function updateDialog($dlg, $newDlg, refreshType, listId) {
+  updateDialog: function updateDialog($dlg, $newDlg, refreshType, listId, doneFunc, dType) {
     $dlg.showNewAfterHide = $newDlg;  // Old dialog will show the new one after being hidden
-    dynamicforms.closeDialog($dlg, refreshType, listId);
+    dynamicforms.closeDialog($dlg, refreshType, listId, doneFunc, dType);
     /*
     // Replace current form with new form containing errors
     var newForm     = $newDlg.find("form");
@@ -471,14 +549,22 @@ dynamicforms = {
    * @param $dlg: dialog to close
    * @param refreshType: how to refresh the table after the dialog is finished with editing
    * @param listId: id of table element with records
+   * @param doneFunc: if specified, this function will be called on successful data send
+   * @param dataType: Custom dataType
    */
-  closeDialog: function closeDialog($dlg, refreshType, listId) {
+  closeDialog: function closeDialog($dlg, refreshType, listId, doneFunc, dataType) {
     if (!dynamicforms.DYNAMICFORMS.jquery_ui) {
+      $dlg.on('hidden.bs.modal', function () {
+        $dlg.remove();
+        if ($dlg.showNewAfterHide)
+          dynamicforms.showDialog($dlg.showNewAfterHide, refreshType, listId, doneFunc, dataType);
+      });
+
       $dlg.modal('hide');
     } else {
       $dlg.remove();
       if ($dlg.showNewAfterHide) {
-        dynamicforms.showDialog($dlg.showNewAfterHide, refreshType, listId);
+        dynamicforms.showDialog($dlg.showNewAfterHide, refreshType, listId, doneFunc, dataType);
       }
     }
   },
@@ -488,17 +574,30 @@ dynamicforms = {
    * @param recordURL: url to call to get data / html for the record / dialog
    * @param refreshType: how to refresh the table
    * @param listId: id of table element with records
+   * @param params: additional parameters in dictionary format
    */
-  editRow: function editRow(recordURL, refreshType, listId) {
+  editRow: function editRow(recordURL, refreshType, listId, params) {
     if (dynamicforms.DYNAMICFORMS.edit_in_dialog) {
+      var ajaxSetts = {
+        url:     recordURL,
+        headers: {'X-DF-RENDER-TYPE': 'dialog'},
+      }
+      if (params != undefined && params.data != undefined)
+        ajaxSetts['data'] = params.data;
+
       dynamicforms.ajaxWithProgress({
-                                      ajax_setts: {
-                                        url:     recordURL,
-                                        headers: {'X-DF-RENDER-TYPE': 'dialog'},
-                                      }
+                                      ajax_setts: ajaxSetts
                                     })
         .done(function (dialogHTML) {
-          dynamicforms.showDialog($(dialogHTML), refreshType, listId);
+          var submitDataType = undefined;
+          var submitDoneFunc = undefined;
+          try {
+            submitDataType = params.submitParams.dataType;
+          } catch (e) {}
+          try {
+            submitDoneFunc = params.submitParams.doneFunc;
+          } catch (e) {}
+          dynamicforms.showDialog($(dialogHTML), refreshType, listId, submitDoneFunc, submitDataType);
         })
         .fail(function (xhr, status, error) {
           // TODO: this doesn't handle errors correctly
@@ -839,18 +938,75 @@ dynamicforms = {
   fieldSetValue: function fieldSetValue(field, value, select2_ajax_option_text) {
     var $field = field instanceof jQuery ? field : dynamicforms.field_helpers.get(field, '$field');
     if ($field.attr('type') == 'checkbox')
-      return $field.prop('checked', value);
+      $field.prop('checked', value);
     if ($field.data('select2')) {
       if ($field.data('select2').options.options.ajax) {
         var opt = $('<option value="' + value + '"></option>').text(select2_ajax_option_text);
         $field.append(opt);
       }
       $field.val(value);
-      $field.trigger('change');
-      return;
+      dynamicforms.setSelect2Order($field, value);
     }
     $field.val(value);
+    $field.trigger('change');
   },
+  setSelect2Order: function setSelect2Order($field, value, forceSet){
+    if(!forceSet)
+      forceSet = false;
+
+    var vals
+    if(forceSet || $field.data('preserved-order') != undefined){
+      if(value.constructor === Array)
+        vals = value;
+      else
+        vals = value.split(',');
+
+      vals     = vals.length == 1 && vals[0] == "" ? [] : vals;
+      $field.data('preserved-order', vals);
+      dynamicforms.select2_renderSelections($field);
+    }
+  },
+  select2_renderSelections: function select2_renderSelections($select2){
+    var order      = $select2.data('preserved-order') || [];
+    var $container = $select2.next('.select2-container');
+    var $tags      = $container.find('li.select2-selection__choice');
+    var $input     = $tags.last().next();
+
+    // apply tag order
+    var i;
+    var val;
+    var $el;
+    for(i = 0; i<order.length; i++){
+      val = order[i];
+      $el = $tags.filter(function(i,tag){
+        return $(tag).data('data').id === val;
+      });
+      $input.before( $el );
+    }
+  },
+
+  selectionHandler: function selectionHandler(e){
+    var $select2  = $(this);
+    var val       = e.params.data.id;
+    var order     = $select2.data('preserved-order') || [];
+    var found_index;
+
+    switch (e.type){
+      case 'select2:select':
+        found_index = order.indexOf(val);
+        if (found_index < 0 )
+          order[ order.length ] = val;
+        break;
+      case 'select2:unselect':
+        found_index = order.indexOf(val);
+        if (found_index >= 0 )
+          order.splice(found_index,1);
+        break;
+    }
+    $select2.data('preserved-order', order); // store it for later
+    dynamicforms.select2_renderSelections($select2);
+  },
+
 
   /**
    * "Standard" function for setting an input's visibility. Any special cases will be handled in custom functions
