@@ -279,3 +279,14 @@ def dict_item_default(context, var, d, k, default):
     context[var] = d.get(k, default)
 
     return ''
+
+
+@register.filter
+def class_name(value):
+    return value.__class__.__name__
+
+
+@register.filter
+def in_list(value, the_list):
+    value = str(value)
+    return value in the_list.split(',')
