@@ -1,7 +1,7 @@
 from django.conf.urls import url
-from dynamicforms import views
+from dynamicforms import progress
 
 urlpatterns = [
     # Progress is used for checking on progress of operation on server
-    url(r'^progress/$', views.get_progress_value, name='progress'),
+    url(r'^progress/$', progress.get_progress_value, name='progress'),
 ]
