@@ -7,6 +7,7 @@ from django.contrib.auth.views import redirect_to_login
 from django.db import models
 from django.http import Http404
 from rest_framework import status, viewsets
+from rest_framework.exceptions import ValidationError
 from rest_framework.response import Response
 from rest_framework.serializers import ListSerializer
 from rest_framework.utils.serializer_helpers import ReturnDict, ReturnList
@@ -54,6 +55,30 @@ class NewMixin(object):
                 raise
 
 
+class PutPostMixin(object):
+    """
+    Provides support for when there is no record id in URL when calling PUT
+    (First empty form is loaded. Than user loads existing data to this form and updates it... - Perform PUT action)
+
+    or
+
+    Provides support for when there is record id in URL when calling POST
+    (First form for some existing record is loaded. Than user wants to create new record for it.
+    - deletes redord ID and perform POST action)
+    """
+
+    # When there is no record id in URL when calling PUT, this function will be called
+    # noinspection PyUnresolvedReferences
+    def put(self: viewsets.ModelViewSet, request, *args, **kwargs):
+        self.kwargs['pk'] = request.data['id']
+        return self.update(request, *args, **kwargs)
+
+    # When there is record id in URL when calling POST, this function will be called
+    # noinspection PyUnresolvedReferences
+    def post(self: viewsets.ModelViewSet, request, *args, **kwargs):
+        return self.create(request, *args, **kwargs)
+
+
 class TemplateRendererMixin():
     template_context = {}
     """
@@ -126,6 +151,7 @@ class TemplateRendererMixin():
                     res.template_name = DYNAMICFORMS.modal_dialog_template
                 elif self.render_type == 'form':
                     serializer.data_template = res.data.serializer.template_name
+                    res.template_name = res.data.serializer.template_name
                 else:
                     if isinstance(serializer, ListSerializer):
                         serializer.child.render_type = 'table'
@@ -140,7 +166,7 @@ class TemplateRendererMixin():
         return res
 
 
-class ModelViewSet(NewMixin, TemplateRendererMixin, viewsets.ModelViewSet):
+class ModelViewSet(NewMixin, PutPostMixin, TemplateRendererMixin, viewsets.ModelViewSet):
     """
     In addition to all the functionality, provided by DRF, DynamicForms ViewSet has some extra features:
 
@@ -160,21 +186,22 @@ class ModelViewSet(NewMixin, TemplateRendererMixin, viewsets.ModelViewSet):
 
         :return: filtered records
         """
-        queryset = super().get_queryset()
-        queryset = self.filter_queryset(queryset)
+        return super().get_queryset()
 
-        return queryset.all()
-
-    def filter_queryset(self, queryset):
+    def filter_queryset(self, queryset, query_params=None):
         """
         Applies filters for all fields
 
         :param queryset: Queryset
+        :param query_params: Custom query_params if needed
         :return: queryset with filters applied
         """
         res = queryset
-        for fld, val in self.request.query_params.items():
-            res = self.filter_queryset_field(res, fld, val)
+        if self.request:
+            if query_params is None:
+                query_params = self.request.query_params
+            for fld, val in query_params.items():
+                res = self.filter_queryset_field(res, fld, val)
         return res
 
     # noinspection PyMethodMayBeStatic
@@ -238,6 +265,16 @@ class ModelViewSet(NewMixin, TemplateRendererMixin, viewsets.ModelViewSet):
 
         return MyCursorPagination
 
+    def create(self, request, *args, **kwargs):
+        try:
+            return super().create(request, *args, **kwargs)
+        except ValidationError as e:
+            instance = self.new_object()
+            ser = self.get_serializer(instance, data=request.data, partial=False)
+            ser.is_valid(raise_exception=False)
+            e.detail.serializer = ser
+            raise e
+
 
 class SingleRecordViewSet(NewMixin, TemplateRendererMixin, viewsets.GenericViewSet):
 
@@ -246,3 +283,9 @@ class SingleRecordViewSet(NewMixin, TemplateRendererMixin, viewsets.GenericViewS
 
     def create(self, request, *args, **kwargs):
         raise NotImplementedError()
+
+
+# noinspection PyUnresolvedReferences
+class GenericViewSet(NewMixin, PutPostMixin, TemplateRendererMixin, viewsets.GenericViewSet):
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
